#import <objectbase/SwarmObject.h>

 
//Macros for bittables
#define WORD(bit)	(bit>>4)
#define MAXCONDBITS	80

/*
id
makeProbe (id obj, const char *ivarName);

double
getDouble (id obj, const char *ivarName);

int
getInt (id obj, const char *ivarName);
*/

@class Interval;

@interface BFParams:  SwarmObject
{
  @public
    int numfcasts; /*"number of forecasts maintained by this agent"*/
  int condwords; /*"number of words of memory required to hold bits"*/
  int condbits; /*"number of conditions bits are monitored"*/
  int mincount; /*"minimum number of times forecast must be used to become active"*/
  int gafrequency; /*"how often is genetic algorithm done?"*/
  int firstgatime; /*"after how many time steps is the genetic algorithm done"*/
  int longtime;	/*" unused time before Generalize() in genetic algorithm"*/
  int individual;
  Interval *tauv;
  Interval *lambda;
  Interval *maxbid;
  Interval *bitprob;
  Interval *subrange;	/*" fraction of min-max range for initial random values"*/
  Interval *a_min, *a_max;	/*" min and max for p+d coef"*/
  Interval *b_min, *b_max;	/*" min and max for div coef"*/
  Interval *c_min, *c_max;	/*" min and max for constant term"*/
  Interval *a_range, *b_range, *c_range;	/*" derived: max - min" */
  Interval *newfcastvar;	/*" variance assigned to a new forecaster"*/
  Interval *initvar;	/*" variance of overall forecast for t<200"*/
  Interval *bitcost;	/*" penalty parameter for specificity"*/
  Interval *maxdev;	/*" max deviation of a forecast in variance estimation"*/
  Interval *poolfrac;	/*" fraction of rules in replacement pool"*/
  Interval *newfrac;	/*" fraction of rules replaced"*/
  Interval *pcrossover;	/*" probability of running Crossover()."*/
  Interval *plinear;	/*" linear combination "crossover" prob."*/
  Interval *prandom;	/*" random from each parent crossover prob."*/
  Interval *pmutation;	/*" per bit mutation prob."*/
  Interval *plong;	        /*" long jump prob."*/
  Interval *pshort;	/*" short (neighborhood) jump prob."*/
  Interval *nhood;	        /*" size of neighborhood."*/
  Interval *genfrac;	/*" fraction of 0/1 bits to generalize"*/
  Interval *gaprob;	/*" derived: 1/gafrequency"*/
  int npool;		/*" derived: replacement pool size"*/
  int nnew;		/*" derived: number of new rules"*/
  int nnulls;            /*" unnused bits"*/
  int *bitlist;		/*" dynamic array, length condbits"*/
  Interval **problist;	/*" dynamic array, length condbits"*/
 
  int npoolmax ;		/* size of reject array */
  int nnewmax ;		/* size of newfcast array */
  int ncondmax;		/* size of newc*/
  int save_size;

  // Variables for loading in floating point parameters only (GP)
@private
  double d_tauv;
  double d_lambda;
  double d_maxbid;
  double d_bitprob;
  double d_subrange;
  double d_a_min, d_a_max;
  double d_b_min, d_b_max;
  double d_c_min, d_c_max;
  double d_newfcastvar;
  double d_initvar;
  double d_bitcost;
  double d_maxdev;
  double d_poolfrac;
  double d_newfrac;
  double d_pcrossover;
  double d_plinear;
  double d_prandom;
  double d_pmutation;
  double d_plong;
  double d_pshort;
  double d_nhood;
  double d_genfrac;
};

+createBegin: aZone;
- init;

- (int*)getBitListPtr;
- (void)copyBitList: (int *) p Length: (int) size;


- (Interval **)getProbListPtr;
- (void)copyProbList: (Interval **) p Length: (int) size;
- (BFParams *) copy: (id <Zone>) aZone;

-(void)drop;

@end




