//#import <objectbase.h>    //Specialist is a SwarmObject
#import "World.h"

@class Agent;

typedef enum 
{
  SP_RE = 0,
  SP_SLOPE = 1,
  SP_ETA = 2,
  SP_RN_START_SLOPE = 3
} SpecialistType;

@class Interval;

@interface Specialist: SwarmObject
{
  Interval * maxprice; /*"Ceiling on stock price"*/
  Interval * minprice; /*"Floor under stock price"*/
  Interval * eta;  /*"Used in adjusting price to balance supply/demand"*/
  // double etainitial; /not used in ASM-2.0
  Interval * minexcess; /*"excess demand must be smaller than this if the price adjustment process is to stop"*/
  Interval * rea; /*"rational expectations benchmark"*/ 
  Interval * reb; /*" trialprice = rea*dividend + reb "*/
  Interval * bidfrac; /*"used in completing trades: volume/bidtotal"*/
  Interval * offerfrac; /*"used in completing trades: volume/offertotal"*/
  int maxiterations; /*" maximum passes while adjusting trade conditions"*/
  //  id agentList; /*" set of traders whose demands must be reconciled"*/
  Interval * volume; /*" volume of trades conducted"*/
  Interval * taupdecay; /*"The agent's profit is calculated as an exponentially weighted moving average.  This coefficient weights old inputs in the EWMA"*/ 
  Interval * taupnew; /*"Used in calculating exponentially weighted moving average;  taupnew = -expm1(-1.0/aTaup); taupdecay =  1.0 - taupnew; "*/ 
  @private
    //   World * worldForSpec; /*" reference to World object that keeps data"*/
  SpecialistType sptype; /*" an enumerated type indicating the sort of Specialist is being used, valued 0, 1, or 2"*/
}
+createBegin: zone;
+create: zone;

// Methods to set parameters
- setMaxPrice: (Interval *)maximumPrice;
- setMinPrice: (Interval *)minimumPrice;
- setTaup: (Interval *)aTaup;
- setSPtype: (int)i;
- setMaxIterations: (int)someIterations;
- setMinExcess: (Interval *)minimumExcess;
- setETA: (Interval *)ETA;

- setREA: (Interval *)REA;
- setREB: (Interval *)REB;


- (Interval *)performTrading: (id)agentList Market: (id)worldForSpec;
- (Interval *)getVolume;
-(double)getVolumeFDOX;
-(double)getVolumeFDON;
- completeTrades: agentList Market: worldForSpec;

-(void)drop;

@end






